@@ -1,4 +1,4 @@
-/*      $Id: config_file.c,v 5.8 2001/11/03 20:44:15 lirc Exp $      */
+/*      $Id: config_file.c,v 5.11 2002/07/01 18:54:01 lirc Exp $      */
 
 /****************************************************************************
  ** config_file.c ***********************************************************
@@ -216,7 +216,7 @@ int checkMode(int is_mode, int c_mode, char *error)
 	{
 		logprintf(LOG_ERR,"fatal error in configfile line %d:",
 			  line);
-		logprintf(LOG_ERR,"\"%s\" isnt valid at this position",
+		logprintf(LOG_ERR,"\"%s\" isn't valid at this position",
 			  error);
 		parse_error=1;
 		return(0);
@@ -352,6 +352,10 @@ int defineRemote(char * key, char * val, char *val2, struct ir_remote *rem)
 		rem->toggle_bit=s_strtoi(val);
 		return(1);
 	}
+	else if (strcasecmp("toggle_mask",key)==0){
+		rem->toggle_mask=s_strtocode(val);
+		return(1);
+	}
 	/* obsolete name */
 	else if (strcasecmp("repeat_bit",key)==0){
 		rem->toggle_bit=s_strtoi(val);
@@ -522,6 +526,14 @@ struct ir_remote * read_config(FILE *f)
                                                 rem->next=s_malloc(sizeof(struct ir_remote));;
                                                 rem=rem->next;
                                         }
+				}else if(mode==ID_codes){
+					add_void_array(&codes_list, defineCode(key, val, &name_code));
+					if(!parse_error && val2!=NULL)
+					{
+						logprintf(LOG_WARNING,"garbage after '%s'"
+							  " code in line %d ignored",
+							  key,line);
+					}
                                 }else{
                                         logprintf(LOG_ERR,"error in configfile line %d:",line);
 					logprintf(LOG_ERR,"unknown section \"%s\"",val);
@@ -586,6 +598,14 @@ struct ir_remote * read_config(FILE *f)
 					{
 						logprintf(LOG_WARNING,"repeat_gap will be ignored if CONST_LENGTH flag is set");
 					}
+				}else if(mode==ID_codes){
+					add_void_array(&codes_list, defineCode(key, val, &name_code));
+					if(!parse_error && val2!=NULL)
+					{
+						logprintf(LOG_WARNING,"garbage after '%s'"
+							  " code in line %d ignored",
+							  key,line);
+					}
                                 }else{
                                         logprintf(LOG_ERR,"error in configfile line %d:",line);
 					logprintf(LOG_ERR,"unknown section %s",val);
