@@ -1,4 +1,4 @@
-/*      $Id: lircd.c,v 5.36 2001/12/08 15:07:03 lirc Exp $      */
+/*      $Id: lircd.c,v 5.46 2002/09/21 15:25:28 lirc Exp $      */
 
 /****************************************************************************
  ** lircd.c *****************************************************************
@@ -87,7 +87,7 @@ static char *repeat_message=NULL;
 
 extern struct hardware hw;
 
-char *progname="lircd-"VERSION;
+char *progname="lircd " VERSION;
 char *configfile=LIRCDCFGFILE;
 char *logfile=LOGFILE;
 FILE *pidfile;
@@ -439,6 +439,7 @@ void add_client(int sock)
 	int fd;
 	int clilen;
 	struct sockaddr client_addr;
+	int flags;
 
 	clilen=sizeof(client_addr);
 	fd=accept(sock,(struct sockaddr *)&client_addr,&clilen);
@@ -457,6 +458,11 @@ void add_client(int sock)
 		return;
 	}
 	nolinger(fd);
+	flags=fcntl(fd,F_GETFL,0);
+	if(flags!=-1)
+	{
+		fcntl(fd,F_SETFL,flags|O_NONBLOCK);
+	}
 	if(client_addr.sa_family==AF_UNIX)
 	{
 		cli_type[clin]=CT_LOCAL;
@@ -563,6 +569,7 @@ void connect_to_peers()
 	struct	hostent *host;
 	struct	sockaddr_in	addr;
 	struct timeval now;
+	int enable=1;
 	
 	gettimeofday(&now,NULL);
 	for(i=0;i<peern;i++)
@@ -587,10 +594,12 @@ void connect_to_peers()
 				continue;
 			}
 			
+			(void) setsockopt(peers[i]->socket,SOL_SOCKET,
+					  SO_KEEPALIVE,&enable,sizeof(enable));
+			
 			addr.sin_family=host->h_addrtype;;
 			addr.sin_addr=*((struct in_addr *)host->h_addr);
 			addr.sin_port=htons(peers[i]->port);
-			
 			if(connect(peers[i]->socket,(struct sockaddr *) &addr,
 				   sizeof(addr))==-1)
 			{
@@ -605,6 +614,8 @@ void connect_to_peers()
 				peers[i]->socket=-1;
 				continue;
 			}
+			logprintf(LOG_NOTICE, "connected to %s",
+				  peers[i]->host);
 			peers[i]->connection_failure=0;
 		}
 	}
@@ -957,9 +968,9 @@ void dosigalrm(int sig)
 }
 
 int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
-	     struct ir_ncode **code,int n)
+	     struct ir_ncode **code,int *reps,int n)
 {
-	char *name=NULL,*command=NULL;
+	char *name=NULL,*command=NULL,*repeats,*end_ptr=NULL;
 
 	*remote=NULL;
 	*code=NULL;
@@ -981,6 +992,29 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 		return(send_error(fd,message,"unknown command: \"%s\"\n",
 				  command));
 	}
+	if(reps!=NULL)
+	{
+		repeats=strtok(NULL,WHITE_SPACE);
+		if (repeats!=NULL)
+		{
+			*reps=strtol(repeats,&end_ptr,10);
+			if (*end_ptr || *reps<0 )
+			{
+				return(send_error(fd,message,
+						  "bad send packet\n"));
+			}
+			if (*reps>REPEAT_MAX)
+			{
+				return(send_error
+				       (fd,message,
+					"too many repeats: \"%d\"\n",*reps));
+			}
+		}
+		else
+		{
+			*reps=-1;
+		}
+	}
 	if(strtok(NULL,WHITE_SPACE)!=NULL)
 	{
 		return(send_error(fd,message,"bad send packet\n"));
@@ -1168,8 +1202,8 @@ int list(int fd,char *message,char *arguments)
 	struct ir_remote *remote;
 	struct ir_ncode *code;
 
-	if(parse_rc(fd,message,arguments,&remote,&code,0)==0) return(0);
-
+	if(parse_rc(fd,message,arguments,&remote,&code,NULL,0)==0) return(0);
+	
 	if(remote==NULL)
 	{
 		return(send_remote_list(fd,message));
@@ -1196,11 +1230,13 @@ int send_core(int fd,char *message,char *arguments,int once)
 	struct ir_remote *remote;
 	struct ir_ncode *code;
 	struct itimerval repeat_timer;
+	int reps;
 	
 	if(hw.send_mode==0) return(send_error(fd,message,"hardware does not "
 					      "support sending\n"));
 	
-	if(parse_rc(fd,message,arguments,&remote,&code,2)==0) return(0);
+	if(parse_rc(fd,message,arguments,&remote,&code,
+		    once ? &reps:NULL,2)==0) return(0);
 	
 	if(remote==NULL || code==NULL) return(1);
 	if(once)
@@ -1217,6 +1253,10 @@ int send_core(int fd,char *message,char *arguments,int once)
 			return(send_error(fd,message,"already repeating\n"));
 		}
 	}
+	if(has_toggle_mask(remote))
+	{
+		remote->toggle_mask_state=0;
+	}
 	if(remote->toggle_bit>0)
 		remote->repeat_state=
 		!remote->repeat_state;
@@ -1226,7 +1266,7 @@ int send_core(int fd,char *message,char *arguments,int once)
 	}
 	if(once)
 	{
-		remote->repeat_countdown=remote->min_repeat;
+		remote->repeat_countdown=max(remote->min_repeat,reps);
 	}
 	else
 	{
@@ -1275,7 +1315,7 @@ int send_stop(int fd,char *message,char *arguments)
 	struct ir_ncode *code;
 	struct itimerval repeat_timer;
 	
-	if(parse_rc(fd,message,arguments,&remote,&code,2)==0) return(0);
+	if(parse_rc(fd,message,arguments,&remote,&code,NULL,2)==0) return(0);
 	
 	if(remote==NULL || code==NULL) return(1);
 	if(repeat_remote && repeat_code &&
@@ -1298,6 +1338,7 @@ int send_stop(int fd,char *message,char *arguments)
 		
 		setitimer(ITIMER_REAL,&repeat_timer,NULL);
 		
+		repeat_remote->toggle_mask_state=0;
 		repeat_remote=NULL;
 		repeat_code=NULL;
 		/* clin!=0, so we don't have to deinit hardware */
@@ -1504,7 +1545,7 @@ void free_old_remotes()
 }
 
 
-int waitfordata(unsigned long maxusec)
+int waitfordata(long maxusec)
 {
 	fd_set fds;
 	int maxfd,i,ret,reconnect;
@@ -1598,6 +1639,9 @@ int waitfordata(unsigned long maxusec)
 				tv.tv_sec=0;
 				tv.tv_usec=maxusec;
 			}
+#ifdef SIM_REC
+			ret=select(maxfd+1,&fds,NULL,NULL,NULL);
+#else
 			if(timerisset(&tv) || reconnect)
 			{
 				ret=select(maxfd+1,&fds,NULL,NULL,&tv);
@@ -1606,6 +1650,7 @@ int waitfordata(unsigned long maxusec)
 			{
 				ret=select(maxfd+1,&fds,NULL,NULL,NULL);
 			}
+#endif
 			if(ret==-1 && errno!=EINTR)
 			{
 				logprintf(LOG_ERR,"select() failed");
@@ -1786,6 +1831,7 @@ int main(int argc,char **argv)
 				hw_print_drivers(stderr);
 				exit (EXIT_FAILURE);
 			}
+			break;
 		case 'd':
 			hw.device=optarg;
 			break;
@@ -1840,14 +1886,13 @@ int main(int argc,char **argv)
 		return(EXIT_FAILURE);
 	}
 	
-#ifdef LIRC_NETWORK_ONLY
-	if(peern==0)
+	if(strcmp(hw.name, "null")==0 && peern==0)
 	{
 		fprintf(stderr,"%s: there's no hardware I can use and "
 			"no peers are specified\n",progname);
 		return(EXIT_FAILURE);
 	}
-#endif
+
 	signal(SIGPIPE,SIG_IGN);
 	
 	start_server(permission,nodaemon);
