@@ -1,4 +1,4 @@
-/*      $Id: receive.c,v 5.9 2001/11/18 14:14:24 lirc Exp $      */
+/*      $Id: receive.c,v 5.15 2002/09/21 15:25:28 lirc Exp $      */
 
 /****************************************************************************
  ** receive.c ***************************************************************
@@ -14,6 +14,8 @@
 # include <config.h>
 #endif
 
+#include <limits.h>
+
 #include "hardware.h"
 #include "lircd.h"
 #include "receive.h"
@@ -28,7 +30,7 @@ inline lirc_t lirc_t_max(lirc_t a,lirc_t b)
 	return(a>b ? a:b);
 }
 
-lirc_t get_next_rec_buffer(unsigned long maxusec)
+lirc_t get_next_rec_buffer(lirc_t maxusec)
 {
 	if(rec_buffer.rptr<rec_buffer.wptr)
 	{
@@ -43,8 +45,12 @@ lirc_t get_next_rec_buffer(unsigned long maxusec)
 		{
 			lirc_t data;
 			
-			if(!waitfordata(maxusec)) return(0);
-			data=hw.readdata();
+			data=hw.readdata(2*maxusec<50000 ? 50000:2*maxusec);
+			if(!data)
+			{
+				return 0;
+			}
+
                         rec_buffer.data[rec_buffer.wptr]=data;
                         if(rec_buffer.data[rec_buffer.wptr]==0) return(0);
                         rec_buffer.sum+=rec_buffer.data[rec_buffer.rptr]
@@ -65,12 +71,21 @@ lirc_t get_next_rec_buffer(unsigned long maxusec)
 	return(0);
 }
 
-void init_rec_buffer()
+void init_rec_buffer(void)
 {
 	memset(&rec_buffer,0,sizeof(rec_buffer));
 }
 
-int clear_rec_buffer()
+void rewind_rec_buffer(void)
+{
+	rec_buffer.rptr=0;
+	rec_buffer.too_long=0;
+	rec_buffer.pendingp=0;
+	rec_buffer.pendings=0;
+	rec_buffer.sum=0;
+}
+
+int clear_rec_buffer(void)
 {
 	int move,i;
 
@@ -123,33 +138,20 @@ int clear_rec_buffer()
 			rec_buffer.wptr=0;
 		}
 		
-		data=hw.readdata();
+		data=hw.readdata(0);
 		
 		LOGPRINTF(3,"c%lu",(unsigned long) data&(PULSE_MASK));
 		
 		rec_buffer.data[rec_buffer.wptr]=data;
 		rec_buffer.wptr++;
 	}
-	rec_buffer.rptr=0;
-	
-	rec_buffer.too_long=0;
+
+	rewind_rec_buffer();
 	rec_buffer.is_biphase=0;
-	rec_buffer.pendingp=0;
-	rec_buffer.pendings=0;
-	rec_buffer.sum=0;
 	
 	return(1);
 }
 
-void rewind_rec_buffer()
-{
-	rec_buffer.rptr=0;
-	rec_buffer.too_long=0;
-	rec_buffer.pendingp=0;
-	rec_buffer.pendings=0;
-	rec_buffer.sum=0;
-}
-
 inline void unget_rec_buffer(int count)
 {
 	if(count==1 || count==2)
@@ -164,11 +166,11 @@ inline void unget_rec_buffer(int count)
 	}
 }
 
-inline lirc_t get_next_pulse()
+inline lirc_t get_next_pulse(lirc_t maxusec)
 {
 	lirc_t data;
 
-	data=get_next_rec_buffer(0);
+	data=get_next_rec_buffer(maxusec);
 	if(data==0) return(0);
 	if(!is_pulse(data))
 	{
@@ -178,11 +180,11 @@ inline lirc_t get_next_pulse()
 	return(data&(PULSE_MASK));
 }
 
-inline lirc_t get_next_space()
+inline lirc_t get_next_space(lirc_t maxusec)
 {
 	lirc_t data;
 
-	data=get_next_rec_buffer(0);
+	data=get_next_rec_buffer(maxusec);
 	if(data==0) return(0);
 	if(!is_space(data))
 	{
@@ -199,14 +201,14 @@ int expectpulse(struct ir_remote *remote,int exdelta)
 	
 	if(rec_buffer.pendings>0)
 	{
-		deltas=get_next_space();
+		deltas=get_next_space(rec_buffer.pendings);
 		if(deltas==0) return(0);
 		retval=expect(remote,deltas,rec_buffer.pendings);
 		if(!retval) return(0);
 		rec_buffer.pendings=0;
 	}
 	
-	deltap=get_next_pulse();
+	deltap=get_next_pulse(rec_buffer.pendingp+exdelta);
 	if(deltap==0) return(0);
 	if(rec_buffer.pendingp>0)
 	{
@@ -229,14 +231,14 @@ int expectspace(struct ir_remote *remote,int exdelta)
 
 	if(rec_buffer.pendingp>0)
 	{
-		deltap=get_next_pulse();
+		deltap=get_next_pulse(rec_buffer.pendingp);
 		if(deltap==0) return(0);
 		retval=expect(remote,deltap,rec_buffer.pendingp);
 		if(!retval) return(0);
 		rec_buffer.pendingp=0;
 	}
 	
-	deltas=get_next_space();
+	deltas=get_next_space(rec_buffer.pendings+exdelta);
 	if(deltas==0) return(0);
 	if(rec_buffer.pendings>0)
 	{
@@ -357,7 +359,7 @@ inline lirc_t sync_rec_buffer(struct ir_remote *remote)
 	lirc_t deltas,deltap;
 	
 	count=0;
-	deltas=get_next_space();
+	deltas=get_next_space(1000000);
 	if(deltas==0) return(0);
 	
 	if(last_remote!=NULL && !is_rcmm(remote))
@@ -366,9 +368,9 @@ inline lirc_t sync_rec_buffer(struct ir_remote *remote)
 		      (100-last_remote->eps)/100 &&
 		      deltas<last_remote->remaining_gap-last_remote->aeps)
 		{
-			deltap=get_next_pulse();
+			deltap=get_next_pulse(1000000);
 			if(deltap==0) return(0);
-			deltas=get_next_space();
+			deltas=get_next_space(1000000);
 			if(deltas==0) return(0);
 			count++;
 			if(count>REC_SYNC) /* no sync found, 
@@ -387,13 +389,14 @@ inline int get_header(struct ir_remote *remote)
 	if(is_rcmm(remote))
 	{
 		lirc_t deltap,deltas,sum;
-		deltap=get_next_pulse();
+		
+		deltap=get_next_pulse(remote->phead);
 		if(deltap==0)
 		{
 			unget_rec_buffer(1);
 			return(0);
 		}
-		deltas=get_next_space();
+		deltas=get_next_space(remote->shead);
 		if(deltas==0)
 		{
 			unget_rec_buffer(2);
@@ -412,11 +415,25 @@ inline int get_header(struct ir_remote *remote)
 		unget_rec_buffer(1);
 		return(0);
 	}
-	if(!expectspace(remote,remote->shead))
+	/* if this flag is set I need a decision now if this is really
+           a header */
+	if(remote->flags&NO_HEAD_REP)
 	{
-		unget_rec_buffer(2);
-		return(0);
+		lirc_t deltas;
+		
+		deltas=get_next_space(remote->shead);
+		if(deltas!=0)
+		{
+			if(expect(remote,remote->shead,deltas))
+			{
+				return(1);
+			}
+			unget_rec_buffer(2);
+			return(0);
+		}
 	}
+	
+	rec_buffer.pendings=remote->shead;
 	return(1);
 }
 
@@ -459,16 +476,13 @@ inline int get_gap(struct ir_remote *remote,lirc_t gap)
 		LOGPRINTF(2,"space expected");
 		return(0);
 	}
+	unget_rec_buffer(1);
 	if(data<gap*(100-remote->eps)/100 &&
 	   data<gap-remote->aeps)
 	{
 		LOGPRINTF(1,"end of signal not found");
 		return(0);
 	}
-	else
-	{
-		unget_rec_buffer(1);
-	}
 	return(1);	
 }
 
@@ -513,8 +527,10 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		for(i=0;i<bits;i+=2)
 		{
 			code<<=2;
-			deltap=get_next_pulse();
-			deltas=get_next_space();
+			deltap=get_next_pulse(remote->pzero+remote->pone+
+					      remote->ptwo+remote->pthree);
+			deltas=get_next_space(remote->szero+remote->sone+
+					      remote->stwo+remote->sthree);
 			if(deltap==0 || deltas==0) 
 			{
 				logprintf(LOG_ERR,"failed on bit %d",
@@ -551,10 +567,26 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		}
 		return(code);
 	}
-
+	
 	for(i=0;i<bits;i++)
 	{
 		code=code<<1;
+		if(is_goldstar(remote))
+		{
+			if((done+i)%2)
+			{
+				LOGPRINTF(2,"$1");
+				remote->pone=remote->ptwo;
+				remote->sone=remote->stwo;
+			}
+			else
+			{
+				LOGPRINTF(2,"$2");
+				remote->pone=remote->pthree;
+				remote->sone=remote->sthree;
+			}
+		}
+		
 		if(expectone(remote,done+i))
 		{
 			LOGPRINTF(2,"1");
